(function ($) {

  Drupal.behaviors.MoipCT = {
    attach: function(context, settings) {

      // Disable "continue checkout" button when Moip CT is checked or if it
      // starts the page already checked
      $('#edit-commerce-payment-payment-method.form-radios input', context).change(function () {
        if($(this).val() === 'moip_ct|commerce_payment_moip_ct') {
          $('#edit-buttons input.checkout-continue').hide();
          $('#edit-buttons').addClass('moip-ct');
        } else {
          $('#moip-ct-js-form').hide();
          $('#edit-buttons input.checkout-continue').show();
          $('#edit-buttons').removeClass('moip-ct');
        }
      });

      // Validate credit card number on blur
      $('.payment-way.creditcard .number input', context).blur(function () {
        if(moip.creditCard.isValid($(this).val())) {

          var card = moip.creditCard.cardType($(this).val());
          $('.payment-way.creditcard .number .help')
            .empty()
            .removeClass().addClass('help')
            .addClass('payment-icon-creditcard-' + card.brand.toLowerCase())
            .css('width', '60px')
            .css('height', '35px')
            .css('margin', '0 0 -15px');
        } else {
          $('.payment-way.creditcard .number .help').removeClass().addClass('help').removeAttr('style');
        }
      });

      // Limit credit card security code lenght
      $('.payment-way.creditcard .securitycode input', context).keyup(function () {
        var securitycode = $(this).val();
        if(securitycode.length > 4) {
          $(this).val(securitycode.substring(0, securitycode.length - 1));
        }
      });

      // Manages the exibition of payment way forms
      $('#edit-commerce-payment-payment-details-moip-ct-payment-way .form-radio', context).change(function () {
        $("#moip-ct-messages").empty().hide();
        $('#moip-ct-buttons').show();
        $('#moip-ct-js-form .payment-way').hide();
        $('#moip-ct-js-form .payment-way.' + this.value).show();
        if(this.value === 'creditcard') {
          $('#moip-ct-submit').text(Drupal.t('Pay with credit card'));
        } else if(this.value === 'bankbillet') {
          $('#moip-ct-submit').text(Drupal.t('Pay with bank billet'));
        } else if(this.value === 'banktransfer') {
          $('#moip-ct-submit').text(Drupal.t('Pay with bank transfer'));
        }
        var moip_ct_button = $('#moip-ct-buttons').clone();
        $('#moip-ct-buttons').remove();
        $('#edit-buttons .fieldset-wrapper').prepend(moip_ct_button);
        Drupal.Moip.CT.UI.scrollPageToElement('#moip-safety');
      });
    }
  }

  // Disable the original submit button so that we can handle the data
  // and send it to MoIP, submiting the form after via JS
  $('#edit-commerce-payment-payment-details-moip-ct').ready(function () {
    $('#edit-buttons input.checkout-continue').hide();
    $('#edit-buttons').addClass('moip-ct');
    $('#edit-buttons').addClass('moip-ct');
  });

  Drupal.Moip = {};
  Drupal.Moip.CT = {};
  Drupal.Moip.CT.send = function(order_id) {
    try {
      // Disable other payment methods to avoid conflict
      $('#edit-commerce-payment-payment-method.form-radios input:not(:checked)').attr('disabled', 'disabled');

      Drupal.Moip.CT.paymentway = $('#edit-commerce-payment-payment-details-moip-ct-payment-way .form-radio:checked').val();

      // Disable other payment options to avoid multiple submits to MoIP
      $('#edit-commerce-payment-payment-details-moip-ct-payment-way .form-radio:not(:checked)').parents('.form-item.form-type-radio').hide();
      // Hide js form buttons
      $('#moip-ct-buttons').hide();

      switch(Drupal.Moip.CT.paymentway) {
        case 'creditcard':
          Drupal.Moip.CT.sendCreditCard();
          break;
        case 'bankbillet':
          Drupal.Moip.CT.sendBankBillet();
          break;
        case 'banktransfer':
          Drupal.Moip.CT.sendBankTransfer();
          break;
      }
    }
    catch (error) {
      var txt = "Sorry, we have an undefined error in the system. <br/>";
      txt += "Our team was notified and we will fix it asap.";
      if (typeof jdog_error == 'function') { 
        jdog_error('moip_js_error', Drupal.t(error.message));
      }
      Drupal.Moip.CT.UI.errorAlert(txt);
    }
  }
  Drupal.Moip.CT.sendCreditCard = function () {

    // Hide unnecessary elements and show the throbber
    $('#moip-ct-js-form > :not(.ajax-progress-throbber)').hide();
    $('#moip-ct-js-form .ajax-progress-throbber').show();

    var number = $('.payment-way.creditcard .number input').val();
    if(!moip.creditCard.isValid(number)) {
      Drupal.Moip.CT.UI.sendCreditCardErrorAlert(Drupal.t('The informed card number is invalid. Please verify.'));
      return false;
    }

    var security_code = $('.payment-way.creditcard .securitycode input').val();
    if(!moip.creditCard.isSecurityCodeValid(number, security_code)) {
      Drupal.Moip.CT.UI.sendCreditCardErrorAlert(Drupal.t('The informed security code is invalid. Please verify.'));
      return false;
    }

    card = moip.creditCard.cardType(number);
    var paymentwayoption;
    // Save card brand on the form to be processed by Form API
    switch(card.brand) {
      case 'AMEX':
        paymentwayoption = 'AmericanExpress';
        break;
      case 'DINERS':
        paymentwayoption = 'Diners';
        break;
      case 'HIPERCARD':
        paymentwayoption = 'Hipercard';
        break;
      case 'MASTERCARD':
        paymentwayoption = 'Mastercard';
        break;
      case 'VISA':
        paymentwayoption = 'Visa';
        break;
    }
    $('.moip-ct-paymentwayoption').val(paymentwayoption);

    var expiration_date = $('.payment-way.creditcard .expirationdate input').val();
    var name = $('.payment-way.creditcard .name input').val();
    var birthday = $('.payment-way.creditcard .birthday input').val();
    var phone = $('#moip-user-phone').val();
    var cpf = $('.payment-way.creditcard .cpf input').val();

    var settings = {
      "Forma": "CartaoCredito",
      "Instituicao": paymentwayoption,
      "Parcelas": "1",
      "Recebimento": "AVista",
      "CartaoCredito": {
        "Numero": number,
        "Expiracao": expiration_date,
        "CodigoSeguranca": security_code,
        "Portador": {
          "Nome": name,
          "DataNascimento": birthday,
          "Telefone": phone,
          "Identidade": cpf
        }
      }
    };
    MoipWidget(settings);
  };
  Drupal.Moip.CT.sendBankBillet = function () {
    var settings = {
      "Forma": "BoletoBancario"
    };
    MoipWidget(settings);
  };
  Drupal.Moip.CT.sendBankTransfer = function () {

    var option = $('.payment-way.banktransfer input:checked').val();
    if(option === undefined) {
      Drupal.Moip.CT.UI.errorAlert(Drupal.t('You need to specify the bank.'));
      $('#edit-commerce-payment-payment-details-moip-ct-payment-way .form-type-radio').show();
    } else {

      // Hide other options that weren't selected
      $('.payment-way.banktransfer input:not(:checked)').parents('.field-wrapper').hide();
      $('.moip-ct-paymentwayoption').val(option);
      var settings = {
        "Forma": "DebitoBancario",
        "Instituicao": option
      };
      MoipWidget(settings);
    }
  };
  Drupal.Moip.CT.sendSuccesfull = function(data) {

    Drupal.Moip.CT.answer = JSON.stringify(data);
    switch(Drupal.Moip.CT.paymentway) {
      case 'creditcard':
        Drupal.Moip.CT.submitCheckoutForm();
        break;
      case 'bankbillet':
        Drupal.Moip.CT.UI.externalPaymentUrlAlert('bankbillet', data.url);
        break;
      case 'banktransfer':
        Drupal.Moip.CT.UI.externalPaymentUrlAlert('banktransfer', data.url);
        break;
    }

  };
  Drupal.Moip.CT.submitCheckoutForm = function () {
    if(Drupal.Moip.CT.answer.length === 0) {
      if (typeof jdog_error == 'function') { 
        jdog_error('moip_js_error', Drupal.t('The Moip "Answer" came empty'));
      }
    } else {
      $('.moip-ct-answer').val(Drupal.Moip.CT.answer);
      // The following line is fundamental to PCI compliance, avoiding
      // the data to be sent to server
      $('#moip-ct-js-form').remove();
      $('.moip-ct-answer').parents('form').submit();
    }
  }
  Drupal.Moip.CT.sendFailed = function(data) {

    // See more in https://labs.moip.com.br/parametro/erros-js/
    var error_map = {
      'c900': "Forma de pagamento inválida",
      'c901': "Informe a instituição de pagamento",
      'c902': "Informe a quantidade de parcelas (valor entre 1 e 12)",
      'c903': "Tipo de recebimento inválido",
      'c904': "Número de cartão ou Cofre deve ser informado",
      'c905': "Informe o número do cartão ou número do cartão é inválido",
      'c906': "Informe a data de expiração do cartão (no formato MM/AA)",
      'c907': "Informe o código de segurança do cartão ou o código é inválido",
      'c908': "Informe os dados do portador do cartão",
      'c909': "Informe o nome do portador como está no cartão",
      'c910': "Informe a data de nascimento do portador (no formato DD/MM/AAAA)",
      'c911': "Informe o telefone do portador ou o telefone é inválido",
      'c912': "Informe o CPF do portador",
      'c913': "Informe o cofre a ser utilizado",
      'c914': "Informe o token da Instrução",
      'c124': "O valor da parcela deve ser superior a R$5,00",
      'c236': "Este pagamento já foi realizado"
    };
    var errorArr = $.makeArray(data);
    var messages = '';
    $.each(errorArr, function(index, value) {
      var msg = '';
      msg = eval('error_map.c' + value.Codigo);
      if(msg) {
        messages += msg + '<br />';
      }
    });
    if(messages.length === 0) { // deals with other messages not mapped
      $.each(errorArr, function(index, value) {
        messages += value.Mensagem + '<br />';
      });
    }
    Drupal.Moip.CT.UI.sendCreditCardErrorAlert(messages);
  };
  Drupal.Moip.CT.UI = {};
  Drupal.Moip.CT.UI.messageAlert = function(message) {
    $("#moip-ct-messages").empty();
    $("#moip-ct-messages").removeClass();
    $("#moip-ct-messages").append(message);
    $("#moip-ct-messages").show({
      effect: 'highlight',
    });
  };
  Drupal.Moip.CT.UI.errorAlert = function(message) {
    Drupal.Moip.CT.UI.messageAlert(message);
    $("#moip-ct-messages").addClass('error');
    $('#moip-ct-buttons').show();
  };
  Drupal.Moip.CT.UI.infoAlert = function(message) {
    Drupal.Moip.CT.UI.messageAlert(message);
    $("#moip-ct-messages").addClass('info');
  };
  Drupal.Moip.CT.UI.sendCreditCardErrorAlert = function(message) {
    Drupal.Moip.CT.UI.errorAlert(message);
    $('#moip-ct-buttons').show();
    $('.form-item-commerce-payment-payment-details-moip-ct-payment-way').show();
    $('#moip-ct-js-form .payment-way.creditcard').show();
    $('#moip-ct-js-form .ajax-progress-throbber').hide();
    Drupal.Moip.CT.UI.scrollPageToElement('#moip-ct-messages');
  };
  Drupal.Moip.CT.UI.externalPaymentUrlAlert = function(paymentway, url) {
    var message;
    var args = {'!link': url};
    switch(paymentway) {
      case 'bankbillet':
        message = Drupal.t('<p>To complete your payment, <a href="!link" target="_blank">click here</a> to open your billet in another browser window.</p><p>You will be redirected automatically after this.</p>', args);
        break;
      case 'banktransfer':
        message = Drupal.t('<p>To complete your payment, <a href="!link" target="_blank">click here</a> to open your internet banking in another browser window.</p><p>You will be redirected automatically after this.</p>', args);
        break;
    }
    Drupal.Moip.CT.UI.infoAlert(message);
    $('#moip-ct-messages a').click(function () {
      Drupal.Moip.CT.submitCheckoutForm();
    });
  };
  Drupal.Moip.CT.UI.scrollPageToElement = function(selector) {
    $("html, body").animate({ scrollTop: $(selector).offset().top }, 1000);
  }
})(jQuery);