<?php

/**
 * Module settings form.
 */
function moip_settings_form($form, &$form_state) {
  $form = array();
  $form['basic_settings'] = array(
    '#type' => 'fieldset',
    '#title' => t('Moip integration basic settings')
  );
  $form['basic_settings']['moip_server'] = array(
    '#type' => 'radios',
    '#title' => t('MoIP server'),
    '#options' => array(
      1 => t('Sandbox - use for testing, requires a MoIP Sandbox account'),
      0 => t('Production - use for processing real transactions'),
    ),
    '#default_value' => variable_get('moip_server', ''),
    '#required' => TRUE,
  );
  $form['basic_settings']['moip_token'] = array(
    '#type' => 'textfield',
    '#title' => t('MoIP token'),
    '#default_value' => variable_get('moip_token', ''),
    '#required' => TRUE,
  );
  $form['basic_settings']['moip_key'] = array(
    '#type' => 'textfield',
    '#title' => t('MoIP key'),
    '#default_value' => variable_get('moip_key', ''),
    '#required' => TRUE,
  );
  $form['basic_settings']['moip_display_title'] = array(
    '#type' => 'textfield',
    '#title' => t('Display title'),
    '#description' => t('Text used on the checkout page description of the payment method.'),
    '#default_value' => variable_get('moip_display_title', 'MoIP'),
  );
  $form['basic_settings']['moip_detail_messages'] = array(
    '#type' => 'radios',
    '#title' => t('Do you want to specify each line item on the order info at MoIP?'),
    '#description' => t('If not, the info sent to MoIP will only contain the reason(title)'),
    '#options' => array(
      0 => t('No'),
      1 => t('Yes'),
    ),
    '#default_value' => variable_get('moip_detail_messages', 1),
    '#required' => TRUE,
  );
  $form['basic_settings']['moip_debug'] = array(
    '#type' => 'radios',
    '#title' => t('Do you want to debug every action of this module?'),
    '#options' => array(
      0 => t('No'),
      1 => t('Yes'),
    ),
    '#default_value' => variable_get('moip_debug', FALSE),
    '#required' => TRUE,
  );
  $form['advanced_settings'] = array(
    '#type' => 'fieldset',
    '#title' => t('Advanced settings')
  );
  $moip_classes = array('MoipDrupal' => 'MoipDrupal');
  foreach (get_declared_classes() as $class_name) {
    $reflection_class = new ReflectionClass($class_name);
    if ($reflection_class->isSubclassOf('MoipDrupal')) {
      $moip_classes[$class_name] = $class_name;
    }
  }
  $form['advanced_settings']['moip_class'] = array(
    '#type' => 'select',
    '#title' => t('MoIP Class'),
    '#options' => $moip_classes,
    '#description' => t('MoIP PHP Class to use in this site. It is useful if you have some special use case to deal with.'),
    '#default_value' => variable_get('moip_class'),
  );
  $form['advanced_settings']['display'] = array(
    '#type' => 'fieldset',
    '#title' => t('Order display'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE
  );
  $form['advanced_settings']['display']['moip_order_reason_token'] = array(
    '#type' => 'textfield',
    '#title' => t('Order Reason Token'),
    '#description' => t('What title do you want to use for the orders sent to MoIP. The default ist "Order @order_number at @store". <strong>You can use tokens</strong> here(see the tokens browser above).'),
    '#default_value' => variable_get('moip_order_reason_token'),
  );
  $form['advanced_settings']['display']['moip_phone_token'] = array(
    '#type' => 'textfield',
    '#title' => t('Phone Token'),
    '#description' => t('From where do you want to get the "Phone" field data to send to MoIP. <strong>You can use tokens</strong> here(see the tokens browser above).'),
    '#default_value' => variable_get('moip_phone_token'),
  );
  $form['advanced_settings']['display']['moip_name_token'] = array(
    '#type' => 'textfield',
    '#title' => t('Name Token'),
    '#description' => t('From where do you want to get the "Name" field data to send to MoIP. <strong>You can use tokens</strong> here(see the tokens browser above).'),
    '#default_value' => variable_get('moip_name_token'),
  );
  $form['advanced_settings']['display']['moip_birthday_token'] = array(
    '#type' => 'textfield',
    '#title' => t('Birthday Token'),
    '#description' => t('From where do you want to get the "Birthday" field data to send to MoIP. <strong>You can use tokens</strong> here(see the tokens browser above).'),
    '#default_value' => variable_get('moip_birthday_token'),
  );
  $form['advanced_settings']['display']['moip_cpf_token'] = array(
    '#type' => 'textfield',
    '#title' => t('CPF Token'),
    '#description' => t('From where do you want to get the "CPF" field data to send to MoIP. <strong>You can use tokens</strong> here(see the tokens browser above).'),
    '#default_value' => variable_get('moip_cpf_token'),
  );
  $form['advanced_settings']['display']['tokens'] = array(
    '#theme' => 'token_tree',
    '#token_types' => array('commerce-order'),
  );
  $form['advanced_settings']['comission'] = array(
    '#type' => 'fieldset',
    '#title' => t('Comission settings'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE
  );
  $form['advanced_settings']['comission']['moip_comission_reason_1'] = array(
    '#type' => 'textfield',
    '#title' => t('Reason'),
    '#description' => t('Specify the reason for this comission.'),
    '#default_value' => variable_get('moip_comission_reason_1'),
  );
  $form['advanced_settings']['comission']['moip_comission_user_receiver_1'] = array(
    '#type' => 'textfield',
    '#title' => t('Moip User'),
    '#description' => t('Moip account(email) to receive the comission.'),
    '#default_value' => variable_get('moip_comission_user_receiver_1'),
  );
  $form['advanced_settings']['comission']['moip_comission_type_1'] = array(
    '#type' => 'radios',
    '#title' => t('Comission type'),
    '#description' => t('You need to specify if this comission is a fixed amount or a percentual value from order total.'),
    '#options' => array(
      0 => t('Fixed amount'),
      1 => t('Percentual value'),
    ),
    '#default_value' => variable_get('moip_comission_type_1'),
  );
  $form['advanced_settings']['comission']['moip_comission_value_1'] = array(
    '#type' => 'textfield',
    '#title' => t('Value'),
    '#description' => t('Format: "10.0" (for both fixed amount and percentual value)'),
    '#default_value' => variable_get('moip_comission_value_1'),
  );
  return system_settings_form($form);
}

function moip_settings_form_validate($form, &$form_state) {

  // If something was informed about comissions, validate it
  if (!empty($form_state['values']['moip_comission_reason_1'])
    || !empty($form_state['values']['moip_comission_user_receiver_1'])) {

    if (empty($form_state['values']['moip_comission_reason_1'])
      || empty($form_state['values']['moip_comission_user_receiver_1'])
      || !isset($form_state['values']['moip_comission_type_1'])
      || empty($form_state['values']['moip_comission_value_1'])
    ) {
      form_set_error('advanced_settings', t('The comissions seem to be incorrectly configured. Please review.'));
    }
  }
}

/**
 * Payment method callback: submit form.
 */
function moip_ct_submit_form($payment_method, $pane_values, $checkout_pane, $order) {

  $form = array();

  unset($_SESSION['moip']);
  
  $sandbox = variable_get('moip_server');
  $phone_token = variable_get('moip_phone_token', '');
  $name_token = variable_get('moip_name_token', '');
  $birthday_token = variable_get('moip_birthday_token', '');
  $cpf_token = variable_get('moip_cpf_token', '');
  
  if(empty($phone_token)) {
    throw new Exception(t("Order @order_id didn't provide a phone number to integrate with Moip.", array('@order_id' => $order->order_id)));
  }
  
  $moip_payment_token = '';
  if(!empty($order->commerce_customer_billing)) {
    $moip_class = variable_get('moip_class', 'MoipDrupal');
    $moipDrupal = new $moip_class($order);
    $moip_answer = $moipDrupal->processOrder('moip_ct');
    $moip_payment_token = $moip_answer->__get('token');
    if(empty($moip_payment_token)) {
      throw new Exception(t("Order @order_id didn't received a transaction token from Moip.", array('@order_id' => $order->order_id)));
    }
  }
  else {
    global $user;
    watchdog('error_moip', '<pre>' . print_r($user, TRUE) . '</pre>'
      . '<pre>' . print_r($order, TRUE) . '</pre>'
      . '<pre>' . print_r($pane_values, TRUE) . '</pre>'
      . '<pre>' . print_r($checkout_pane, TRUE) . '</pre>'
    );
    throw new Exception(t("Order @order_id have an empty commerce_customer_billing field.", array('@order_id' => $order->order_id)));
  }

  drupal_add_css(drupal_get_path('module', 'moip') . '/assets/stylesheets/moip.css', 'file');


  $moipjs_path = 'sites/all/libraries/moipjs/moip.min.js';
  if (!file_exists($moipjs_path)) {
    $error = t('Moip JS library was not found on "' . $path . '". Download it in !link', array('!link' => 'https://github.com/moiplabs/moipjs'));
    throw new Exception($error);
  }

  if($sandbox) {
    $js_src = 'https://desenvolvedor.moip.com.br/sandbox/transparente/MoipWidget-v2.js';
  }
  else {
    $js_src = 'https://www.moip.com.br/transparente/MoipWidget-v2.js';
  }
  drupal_add_js($js_src, 'external');

  $form['#attached']['js'][] = array(
    'data' => $moipjs_path,
    'type' => 'file',
  );
  $form['#attached']['js'][] = array(
    'data' => drupal_get_path('module', 'moip') . '/assets/js/moip.min.js',
    'type' => 'file',
  );
  $form['moip_ct'] = array(
    '#type' => 'container'
  );
  $form['moip_ct']['payment_way'] = array(
    '#type' => 'radios',
    '#title' => t('MoIP - Choose your payment way'),
    '#options' => array(
      'creditcard' => '<div class="title">' . t('Credit Card') . '</div><div class="payment-icon payment-icon-creditcard"></div>',
      'bankbillet' => '<div class="title">' . t('Bank Billet') . '</div><div class="payment-icon payment-icon-bankbillet"></div>',
      'banktransfer' => '<div class="title">' . t('Bank Transfer') . '</div><div class="payment-icon payment-icon-banktransfer"></div>',
    ),
    '#required' => TRUE,
  );
  $form['moip_ct']['payment_way_option'] = array(
    '#attributes' => array('class' => 'moip-ct-paymentwayoption'),
    '#type' => 'hidden',
  );
  $form['moip_ct']['answer'] = array(
    '#attributes' => array('class' => 'moip-ct-answer'),
    '#type' => 'hidden',
  );
  $form['moip_ct']['js_form'] = array(
    '#type' => 'markup',
    '#markup' => theme('moip_ct_js_form', array(
      'moip_payment_token' => $moip_payment_token,
      'order_id' => $order->order_id,
      'moip_user_phone' => token_replace($phone_token),
      'moip_user_name' => token_replace($name_token),
      'moip_user_birthday' => token_replace($birthday_token),
      'moip_user_cpf' => token_replace($cpf_token),
    ))
  );

  return $form;
}

function moip_ct_submit_form_submit($payment_method, $pane_form, $pane_values, $order, $charge) {

  $payment_way = $pane_values['moip_ct']['payment_way'];
  $payment_way_option = $pane_values['moip_ct']['payment_way_option'];
  $moip_ct_answer = json_decode($pane_values['moip_ct']['answer']);

  if(variable_get('moip_debug', FALSE)) {
    watchdog('moipdbg_ct_answer', $payment_way_option . ': <pre>' . print_r($moip_ct_answer, TRUE) . '</pre>');
  }

  if(empty($payment_way) || empty($moip_ct_answer)) {
    global $user;
    watchdog('error_moipct', '<pre>' . print_r($user, TRUE) . '</pre>'
      . '<pre>' . print_r($order, TRUE) . '</pre>'
      . '<pre>' . print_r($pane_values, TRUE) . '</pre>'
      . '<pre>' . print_r($moip_ct_answer, TRUE) . '</pre>'
    );
    throw new Exception("Error Processing Request", 1);
  }

  switch($payment_way) {
    case 'creditcard':
    case 'banktransfer':
      if(empty($payment_way_option)) {
        global $user;
        watchdog('error_moipct', '<pre>' . print_r($user, TRUE) . '</pre>'
          . '<pre>' . print_r($order, TRUE) . '</pre>'
          . '<pre>' . print_r($pane_values, TRUE) . '</pre>'
          . '<pre>' . print_r($moip_ct_answer, TRUE) . '</pre>'
        );
        throw new Exception("Error Processing Request", 1);
      }
      break;
  }

  $order->data['moip_ct_payment_method'] = $payment_way;
  $order->data['moip_ct_payment_method_option'] = $payment_way_option;
  commerce_order_save($order);
}